#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-torque-client-config v. 2.1.0
#
# Post-installation script for configuring the gLite torque client
# Guillermo Diez-Andino Sancho <Guillermo.Sancho@cern.ch>
#
# Version info: $Id: glite-torque-client-config.py,v 1.16.2.5.2.2.2.1 2005/11/28 08:40:27 gdiez Exp $
#
# Usage: python glite-torque-client-config [-c|-v|-h|--help|--start|--stop|--status|--configure]
#        -c          print configuration
#        -v          print version
#        -h,--help   print usage info
#        --start     starts the pbs-torque client services (pbs_mom)
#        --stop      stops the pbs-torque  client services (pbs_mom)
#        --status    shows the status of the pbs-torque client services (pbs_mom)
#        --configure configures the pbs-torque client services (pbs_mom)
#        --siteconfig=<site config URL>     use remote site configuration file from URL
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

# import system libraries
import os,string
import sys
import getopt
import time

from gLiteInstallerLib import gLib

# Set global variables here 
global params         			# all config values from the XML file

class glite_torque_client:

	def __init__(self):
		self.verbose = 0
		self.version = "2.1.0"
		self.name = "glite-torque-client"
		self.friendly_name = "gLite Torque Client"
		self.glib = params['glib']
	#-------------------------------------------------------------------------------
	# Banner 
	#-------------------------------------------------------------------------------

	def banner(self):

		print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
		print "          Configuring the %s" % self.friendly_name
		print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
	#-------------------------------------------------------------------------------
	# Copyright 
	#-------------------------------------------------------------------------------

	def copyright(self):

		print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
		print 'See http://eu-egee.org/partners/ for details on the copyright holders'
		print 'For license conditions see the license file or http://eu-egee.org/license.html'

	#-------------------------------------------------------------------------------
	# Version
	#-------------------------------------------------------------------------------

	def showVersion(self):

		print '\n%s-config  v. %s\n' % (self.name,self.version)
    
	#-------------------------------------------------------------------------------
	# Usage
	#-------------------------------------------------------------------------------

	def usage(self,msg = ""):

		if msg:
			print "\n%s" % (msg)
        
		self.copyright()
		self.showVersion()
    
		print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
		os.environ['GLITE_LOCATION'], self.name, self.name)

		print '    -c, --checkconf     print the service configuration'
		print '    -v, --version       print the version of the configuration script'
		print '    -h, --help          print this usage information'
		print '    --start             starts the pbs-torque client services'
		print '    --stop              stops the pbs-torque client services'
		print '    --status            shows the status of the pbs-torque client services (pbs_mom)'
		print '    --configure		   configures the pbs-torque client services (pbs_mom)'
		print '    --siteconfig        URL of the site configuration file'
		print '\n'

	def start(self):
		if self.glib.startProcess('pbs_mom','/etc/rc.d/init.d/pbs_mom start' ) != 0:
			return 1
		return 0

	def stop(self):
		if self.glib.stopProcess('pbs_mom','/etc/rc.d/init.d/pbs_mom stop' ) != 0:
			return 1
		return 0
	
	def status(self):
		retval = os.system('/etc/rc.d/init.d/pbs_mom status')
		if retval != 0:
			error_level = retval
		return 0
	
	def backup(self,path):
		backupPath = "%s.1" % path

 		if os.path.exists(backupPath):
			os.remove(backupPath)
		if os.path.exists(path):
			os.rename(path,backupPath)
		return 0
		
	def existsFile(self,path):
		if os.path.exists(path):
			print "File %s correctly created                 " %path,
			self.glib.printOkMessage()
			return 0
		else:
			print "Problems to create file %s                " %path,
			self.glib.printFailedMessage()
			return 1
	
	def configureSSH(self):
		sshParams = {}
		sshParamsOK = {}
		removedsshParams = {}

		sshParams['Protocol'] = '2,1'
		sshParams['RhostsAuthentication'] = 'yes'
		sshParams['RhostsRSAAuthentication'] = 'yes'
		sshParams['RSAAuthentication'] = 'yes'
		sshParams['PasswordAuthentication'] = 'yes'
		sshParams['EnableSSHKeysign'] = 'yes'
		sshParams['HostbasedAuthentication'] = 'yes'

		sshParamsOK['Protocol'] = '0'
		sshParamsOK['RhostsAuthentication'] = '0'
		sshParamsOK['RhostsRSAAuthentication'] = '0'
		sshParamsOK['RSAAuthentication'] = '0'
		sshParamsOK['PasswordAuthentication'] = '0'
		sshParamsOK['EnableSSHKeysign'] = '0'
		sshParamsOK['HostbasedAuthentication'] = '0'

		inGlobal =1
		inHost = 0
		inHostStar = 0
		firstInHost = 0

		file = open("/etc/ssh/ssh_config",'r')
		tmpFileName = "ssh_config.tmp"
		tmpFile = open(tmpFileName,'w')
		for line in file.xreadlines():
			#Defining the section
			if not line.strip().startswith('#') and line.find('Host *') != -1:
				inGlobal = 0
				inHost = 0
				inHostStar =1
				tmpFile.write(line)
				continue
			else:
				if not line.strip().startswith('#') and line.find('Host ') != -1:
					inGlobal = 0
					inHost = 1
					firstInHost = 1
					inHostStar = 0
					tmpFile.write(line)
					continue
			
			if inGlobal == 1 or inHostStar == 1: #I'm inside HostStar or inGlobal
				iterator = sshParams.iterkeys()
				if not line.strip().startswith('#'):
					while True:
						try:
							element = iterator.next()
							if line.strip().split(" ")[0] == element:
								value = line.strip().split(" ")[1]
								if value == sshParams[element]:
									print "%s %s is already set" %(element,sshParams[element])
									self.glib.printOkMessage()
									tmpFile.write(line)
									sshParamsOK[element] = '1'
									break
								else:
									print "Removing %s %s" %(element,value)
									tmpFile.write("#Removed %s %s by the torque-client deployment module\n" %(element,value))
									removedsshParams[element] = value
									break
						except StopIteration:
							#The value has not been found
							tmpFile.write(line)
							break
				else: #The line is a comment (I write it to the file)
					tmpFile.write(line)			
			else: 						#Inside inHost
				if firstInHost == 1: #First line of a Host section
					firstInHost = 0
					#Add all the values removed from HostStar and Global sections
					iterator = removedsshParams.iterkeys()
					while True:
						try:
							element = iterator.next()
							tmpFile.write("#Line Added by the torque-client deployment module--> %s %s\n" %(element,value))
							tmpFile.write('  %s %s\n' %(element,removedsshParams[element]))
							#print "Adding %s %s" %(element,removedsshParams[element])
						except StopIteration:
							break
				tmpFile.write(line)
		
	
		if inHostStar != 1 and inGlobal !=1:
			tmpFile.write('Host *\n')

		iterator = sshParamsOK.iterkeys()
		while True:
			try:
				element = iterator.next()
				if sshParamsOK[element]  == '0':
					tmpFile.write("#Line Added by the torque-client deployment module--> %s %s\n" %(element,sshParams[element]))
					tmpFile.write('  %s %s\n' %(element,sshParams[element]))
					print "Adding %s %s" %(element,sshParams[element])
					self.glib.printOkMessage()
			except StopIteration:
				break	
		file.close()
		tmpFile.close()
		os.system('mv -f %s /etc/ssh/ssh_config' %tmpFileName)			

	
	#-------------------------------------------------------------------------------
	# All the configuration code goes here
	#-------------------------------------------------------------------------------
	
	def configure(self):
		step = 0
		sshLogFile = "%s/sshLogFile" %os.environ['GLITE_LOCATION_LOG']
		
		#------------------------------------------------------------------------
		#			STOPPING PBS_MOM
		#------------------------------------------------------------------------
		print "%s.- Stopping pbs_mom " %step
		self.stop()
		step+=1
		
		#------------------------------------------------------------------------
		#			CREATING SERVER_NAME FILE
		#------------------------------------------------------------------------
		print "%s.- Adding server name /var/spool/pbs/server_name" %step
		self.backup("/var/spool/pbs/server_name")
		file = open('/var/spool/pbs/server_name','w')
		file.write("%s\n" % params['torque-server.name'])
		file.close()
		if self.existsFile('/var/spool/pbs/server_name') != 0:
			return 1
		step+=1

		#------------------------------------------------------------------------
		#			ADDING PORTS
		#------------------------------------------------------------------------
		print "%s.- Adding ports to /etc/services " %step
		file = open("/etc/services",'r')
		pbsMomTcp = 1
		pbsReMomTcp = 1
		pbsReMomUdp = 1
		for line in file.xreadlines():
			if line.find('pbs_mom         15002/tcp') != -1:
				pbsMomTcp = 0
			if line.find('pbs_remom       15003/tcp') != -1:
				pbsReMomTcp = 0
			if line.find('pbs_remom       15003/udp') != -1:
				pbsReMomUdp = 0	
		file.close()
		if pbsMomTcp == 0 and pbsReMomTcp == 0 and pbsReMomUdp == 0:
			print "The ports already existed in /etc/services                       ",
			self.glib.printOkMessage()
		else:
			file = open('/etc/services','a')
			if pbsMomTcp == 1:	
				file.write('\npbs_mom         15002/tcp')
				print "pbs_mom port added to /etc/services 15002/tcp                                 ",
				self.glib.printOkMessage()
			if pbsReMomTcp == 1:
				file.write('\npbs_remom       15003/tcp')
				print "pbs_remom port added to /etc/services 15003/tcp                               ",
				self.glib.printOkMessage()
			if pbsReMomUdp == 1:
				file.write('\npbs_remom       15003/udp')
				print "pbs_remom port added to /etc/services 15003/udp                               ",
				self.glib.printOkMessage()
			file.close()
		step+=1
	
		#------------------------------------------------------------------------
		#			CREATING THE SSH CONFIGURATION
		#------------------------------------------------------------------------
		print "%s. Checking and adapting the ssh_config configuration file" %step
		self.configureSSH()			
		if self.existsFile('/etc/ssh/ssh_config') != 0:
			return 1
		step+=1
		
		#------------------------------------------------------------------------
		#                       DUPLICATED KEY ENTRIES
		#------------------------------------------------------------------------
		print "%s.Looking for duplicated entries" %step
		scriptFile = "%s/searchKeys.sh" %os.environ['GLITE_LOCATION_TMP']
		self.backup(scriptFile)
		file = open(scriptFile,"w")
                                                                                                                                                   
		file.write("#!/bin/bash\n")
		file.write("for hostname in %s; do\n" % params['torque-server.name'])
		file.write("    if [ -f /etc/ssh/ssh_known_hosts ];then \n")
		file.write("            grep -v $hostname /etc/ssh/ssh_known_hosts  > /etc/ssh/ssh_known_hosts.tmp\n")
		file.write("            /usr/bin/ssh-keyscan -t rsa $hostname >> /etc/ssh/ssh_known_hosts.tmp 2>/dev/null\n")
		file.write("            if [ $? = 0 ]; then\n")
		file.write("                    mv -f /etc/ssh/ssh_known_hosts.tmp  /etc/ssh/ssh_known_hosts\n")
		file.write("            fi\n")
		file.write("    fi\n")
		file.write("done\n")
		file.close()
		os.system("sh %s" %scriptFile)
		step +=1
		
		#------------------------------------------------------------------------
		#			CREATING CONFIGURATION FILE /opt/edg/etc/edg-pbs-knownhosts.conf
		#------------------------------------------------------------------------
		print "%s.- Creating file /opt/edg/etc/edg-pbs-knownhosts.conf" %step
		self.backup("/opt/edg/etc/edg-pbs-knownhosts.conf")
		file = open("/opt/edg/etc/edg-pbs-knownhosts.conf","w")
		file.write("NODES = %s\n" %params['torque-server.name'])
		file.write("PBSBIN = /usr/bin\n")
		file.write("KEYTYPES = rsa1,rsa,dsa\n")
		file.write("KNOWNHOSTS = /etc/ssh/ssh_known_hosts\n")
		file.close()
		if self.existsFile('/opt/edg/etc/edg-pbs-knownhosts.conf') != 0:
			return 1
		step+=1
		
		#------------------------------------------------------------------------
		#			ADDING /opt/edg/etc/edg-pbs-knownhosts.conf TO CRON.D DIRECTORY
		#------------------------------------------------------------------------
		print "%s.- Adding edg-pbs-knownhosts to the cron.d directory" %step
		file = open("/etc/cron.d/edg-pbs-knownhosts","w")
		file.write("03 1,7,13,19 * * * root /opt/edg/sbin/edg-pbs-knownhosts\n")
		file.close()
		if self.existsFile('/etc/cron.d/edg-pbs-knownhosts') != 0:
			return 1
		os.system("chmod +x /etc/cron.d/edg-pbs-knownhosts")
		step+=1
		
		#------------------------------------------------------------------------
		#			RUNNING /opt/edg/sbin/edg-pbs-knownhosts
		#------------------------------------------------------------------------
		print "%s.- Running  /opt/edg/sbin/edg-pbs-knownhosts " %step
		os.system("/opt/edg/sbin/edg-pbs-knownhosts")
		step+=1
		
		if params['mpi.copy.enable'] == "True":
			print "MPI Copy enabled. Configuring sshd  "
			
			#------------------------------------------------------------------------
			#			CREATING /opt/edg/etc/edg-pbs-shostsequiv.conf FILE
			#------------------------------------------------------------------------
			print "%s.- Creating file /opt/edg/etc/edg-pbs-shostsequiv.conf" %step
			self.backup("/opt/edg/etc/edg-pbs-shostsequiv.conf")
			file = open("/opt/edg/etc/edg-pbs-shostsequiv.conf","w")
			file.write("SHOSTSEQUIV = /etc/ssh/shosts.equiv\n")
			file.write("NODES =\n")
			file.write("PBSBIN      = /usr/bin\n")
			file.close()
			if self.existsFile('/opt/edg/etc/edg-pbs-shostsequiv.conf') != 0:
				return 1
			self.glib.printOkMessage()	
			step+=1
		
			#------------------------------------------------------------------------
			#			ADDING edg-pbs-shostequiv.conf to /etc/cron.d directory
			#------------------------------------------------------------------------
			print "%s.- Adding edg-pbs-shostequiv.conf to /etc/cron.d directory" %step
			file = open("/etc/cron.d/edg-pbs-shostsequiv","w")
			file.write("05 1,7,13,19 * * * root /opt/edg/sbin/edg-pbs-shostsequiv\n")
			file.close()
			if self.existsFile('/etc/cron.d/edg-pbs-shostsequiv') != 0:
				return 1
			self.glib.printOkMessage()	
			os.system("chmod +x /etc/cron.d/edg-pbs-shostsequiv")
			step+=1
		
			#------------------------------------------------------------------------
			#			RUNNING edg-pbs-shostsequiv
			#------------------------------------------------------------------------
			print "%s.- Running edg-pbs-shostsequiv " %step
			os.system("/opt/edg/sbin/edg-pbs-shostsequiv")
			self.glib.printOkMessage()
			step+=1
			
			#------------------------------------------------------------------------
			#			CREATING THE SSHD CONFIGURATION
			#------------------------------------------------------------------------
			hostBasedAuth = 1
			ignoreUserKnownHost = 1
			print "%s.- Changing /etc/ssh/sshd_config" %step
			step+=1
			file = open("/etc/ssh/sshd_config",'r')
			for line in file.xreadlines():
				if line.find('HostbasedAuthentication yes') == 0:
					hostBasedAuth = 0
				if line.find('IgnoreUserKnownHosts yes') == 0:
					ignoreUserKnownHost = 0	
			file.close()
				
			if hostBasedAuth == 0 and ignoreUserKnownHost == 0:
				print "sshd_config already configured"
				self.glib.printOkMessage()
			else:	
				file = open('/etc/ssh/sshd_config','a')
				if hostBasedAuth == 1:
					file.write("\nHostbasedAuthentication yes")
					print "Added HostbasedAuthentication yes to /etc/ssh/sshd_config"
					sshdLog = open(sshLogFile,'w')
					self.glib.printOkMessage()
					sshdLog.close()
				if 	ignoreUserKnownHost == 1:
					file.write("\nIgnoreUserKnownHosts yes")
					print "Added IgnoreUserKnownHosts yes to /etc/ssh/sshd_config"
					self.glib.printOkMessage()
				file.close()
				print "sshd_config changed"
				self.glib.printOkMessage()
			
				if self.existsFile('/etc/ssh/sshd_config') != 0:
					return 1
					
				#------------------------------------------------------------------------
				#		RESTARTING SSHD
				#------------------------------------------------------------------------
				print "%s.- Restarting sshd " %step
				os.system("/etc/rc.d/init.d/sshd restart")
				step +=1
		else:
			print "%s.- Checking if the sshd configuration needs to be restored " %step
			step+=1
			
			if os.path.exists('/opt/edg/etc/edg-pbs-shostsequiv.conf'):
				print "%s.- Removing /opt/edg/etc/edg-pbs-shostsequiv.conf " %step
				os.system('rm -rf /opt/edg/etc/edg-pbs-shostsequiv.conf')
				step+=1
				
			if os.path.exists('/etc/cron.d/edg-pbs-shostsequiv'):
				print "%s.- Removing /etc/cron.d/edg-pbs-shostsequiv" %step
				os.system('rm -rf /etc/cron.d/edg-pbs-shostsequiv')
				step+=1
				
			if os.path.exists(sshLogFile):
				if os.path.exists('/etc/ssh/shosts.equiv'):
					print "%s.- Removing /etc/ssh/shosts.equiv" %step
					os.system('rm -rf /etc/ssh/shosts.equiv')
					step+=1
			
				print "%s.- Looking for HostbasedAuthentication in /etc/sshd/sshd_config to be removed" %step
				os.system('cat /etc/ssh/sshd_config | grep -v \'^HostbasedAuthentication yes\' | grep -v \'^IgnoreUserKnownHosts yes\' > %s/output.tmp' %os.environ['GLITE_LOCATION_LOG'])
				os.system('mv -f %s/output.tmp /etc/ssh/sshd_config' %os.environ['GLITE_LOCATION_LOG'])
				step+=1
				
				print "%s.- Restarting sshd" %step
				os.system('/etc/rc.d/init.d/sshd restart')
				step+=1
				
				print "%s.- Removing temporary files" %step
				if os.path.exists(sshLogFile):
					print "-->Removing %s" %sshLogFile
					os.system('rm -rf %s' %sshLogFile)				
				step+=1
						
		#------------------------------------------------------------------------
		#			PBS_MOM CONFIGURATION
		#------------------------------------------------------------------------
		print "%s.- Creating the pbs_mom configuration file " %step
		self.backup("/var/spool/pbs/mom_priv/config")
		file = open("/var/spool/pbs/mom_priv/config","w")
		file.write("$clienthost %s\n" %params['torque-server.name'])
		file.write("$clienthost localhost\n")
		file.write("$restricted %s\n" %params['torque-server.name'])
		file.write("$logevent %s\n" %params['mom-server.logevent'])
		file.write("$loglevel %s\n" %params['mom-server.loglevel'])
		file.write("$ideal_load 1.6\n")
		file.write("$max_load 2.1\n")
		file.close()
		if self.existsFile("/var/spool/pbs/mom_priv/config") != 0:
			return 1
		self.glib.printOkMessage()	
		step+=1
		
		#------------------------------------------------------------------------
		#			RESTARTING PBS_MOM
		#------------------------------------------------------------------------
		#print "%s.- Starting pbs_mom" %step
		#os.system("/sbin/chkconfig pbs_mom on")
		#if self.glib.stopProcess('pbs_mom','/etc/rc.d/init.d/pbs_mom stop') != 0:
	    #	return 1
		#os.system("sleep 1")
		#if self.glib.startProcess('pbs_mom','/etc/rc.d/init.d/pbs_mom start') != 0:
		#	return 1
		#self.glib.printOkMessage()	
		#step+=1
		
		return 0
	
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():

	glib.export('GLITE_LOCATION');
	glib.export('GLITE_LOCATION_VAR');
	if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
		os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
	glib.export('GLITE_LOCATION_LOG');
	if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
		os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
	glib.export('GLITE_LOCATION_TMP');
	if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
		os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

	glib.setUserEnv()	
	return 0
	
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

	# The script must be run as root
	if not os.geteuid()==0:
		print '"[FAILED] This script must be run as root'
		sys.exit(1)
	
	# Get an instance of the library class
	glib = gLib()
	
	# Load parameters
	params = {} 
	
	params['glib'] = glib
	try:
		opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
		for o, a in opts:
			if o == "--siteconfig":
				params['site.config.url'] = a
				break
	except getopt.GetoptError:
		pass
	glib.loadConfiguration("%s/../glite-torque-client.cfg.xml" % glib.getScriptPath(), params) 	

	# set verbosity
	if params.has_key('glite.installer.verbose'):
		if (params['glite.installer.verbose'] == "true"):
			verbose=1
		else:
			verbose=0
	else:
		verbose = 0
	glib.verbose = verbose
	
	# Set up the environment
    	service = glite_torque_client()
	set_env()	
                                                                                                  
        path = '%s/etc' % os.environ['GLITE_LOCATION']
        if not os.path.exists(path):
           print '[FAILED] %s/etc does not exist! Please verify your gLite installation' % os.environ['GLITE_LOCATION']
           sys.exit (1)
        else:
           print '[OK] %s/etc exists' % os.environ['GLITE_LOCATION']

        # Command line opts if any
	try:
		opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version','stop','start','status','configure','subservice','siteconfig='])
	except getopt.GetoptError:
		service.usage(msg = "[FAILED] Unknown options(s)")
		sys.exit(1)

	# Check cli options
	configure = 0
	subservice = 0
	for o, a in opts:
		if o in ("-h", "--help"):
			service.usage()
			sys.exit(0)
		if o in ("-v", "--version"):
			service.showVersion()
			sys.exit(0)
		if o in ("-c", "--checkconf"):
			service.copyright()
			service.showVersion()
			glib.print_params(params)
			sys.exit(0)
   		if o == "--start":
			if service.start() == 0:
				print "[OK] The %s was successfully started\n" % service.friendly_name
				sys.exit(0)
			else:
				print "[FAILED] An error occurred while starting the %s service" % service.friendly_name
				sys.exit(1)
		if o == "--stop":
			if service.stop() == 0:
				print "[OK] The %s was successfully stopped\n" % service.friendly_name
				sys.exit(0)
			else:
				print "[FAILED] An error occurred while stopping the %s service" % service.friendly_name
				sys.exit(1)
		if o == "--status":
			if service.status() == 0:
				print "[OK] The %s status was successfully showed\n" % service.friendly_name
				sys.exit(0)
			else:
				print "[FAILED] An error occurred while showing the %s service status" % service.friendly_name
				sys.exit(1)
		if o == "--subservice":
			subservice = 1		
		if o == "--configure":
			configure =1
	
	if configure == 1:
		if subservice == 0:
			service.copyright()
			service.showVersion()
		# Configure the service
		if service.configure() == 0:
			print "%s configuration successfully completed              " % service.friendly_name,
			glib.printOkMessage()
			glib.registerService()
			sys.exit(0)
		else:
			print "An error occurred while configuring the %s service   " % service.friendly_name,
			glib.printFailedMessage()
			sys.exit(1)
	else:		
 		# Print configuration parameters
		if verbose:
			glib.print_params(params)
			service.copyright()
			service.showVersion()
			service.banner()
			service.usage()
